# Template data example:
# [
#     {
#         "translation_domain": "acl",
#         "sourcepackage": "acl",
#         "template_name": "acl",
#         "enabled": true,
#         "languagepack": true,
#         "priority": 100,
#         "date_last_updated": "2008-01-12T09:20:55.871212",
#         "total": 51
#     }
# ]
#
# Translation data example:
# [
#     {
#        "translation_domain": "debian-installer",
#        "sourcepackage": "debian-installer",
#        "language": "aa",
#        "template_name": "debian-installer",
#        "changed": 0,
#        "template_last_updated": "2008-07-27T17:27:48.802558",
#        "unreviewed": 0,
#        "translated": 148,
#        "total": 1731,
#        "translation_last_updated": "2007-03-02T21:21:04.653133"
#     }
# ]

from django.core.management.base import NoArgsCommand
from django.db import transaction
from django.db.models import Sum
from django.conf import settings
from django.db import connection
from common import launchpad
from utils import distropackages
from stats.models import UbuntuRelease, Language, Timestamp, Template, \
                         Translation
from django.conf import settings
import shutil
import atexit
import tempfile
import urllib
import os
import gzip
import json
import sys
import datetime
import time


class Command(NoArgsCommand):
    help = "Get the translations data and populate the database with it"

    def __init__(self):
        self.CONFIG_PATH = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'config')

    def calc_coverage(self, release, language):

        translated = \
            Translation.objects.filter(
                language=language,
                template__release=release,
                template__enabled=True,
                template__whitelisted=True,
                template__blacklisted=False,
                template__priority__gt=settings.MIN_TEMPLATE_PRIORITY
                                       ).aggregate(Sum('translated'))

        total = \
            Template.objects.filter(
                release=release,
                enabled=True,
                whitelisted=True,
                blacklisted=False,
                priority__gt=settings.MIN_TEMPLATE_PRIORITY
                                    ).aggregate(Sum('total'))

        translated_sum = translated['translated__sum']
        total_sum = total['total__sum']

        if not translated_sum:
            coverage = 0
            if not total_sum:
                untranslated = 0
            else:
                untranslated = total_sum
        elif not total_sum:
            coverage = 0
            untranslated = 0
        else:
            coverage = translated_sum * 100.0 / total_sum 
            untranslated = total_sum - translated_sum

        return coverage, untranslated

    @transaction.commit_manually
    def add_timestamp(self):
        releases = UbuntuRelease.objects.all()
        languages = Language.objects.all()
        timeisnow = datetime.datetime.now()

        for release in releases:
            for language in languages:
                if language.visible:
                    coverage, untranslated = self.calc_coverage(release,
							language)
                    timestamp = Timestamp(release=release,
                                          timestamp=timeisnow,
                                          language=language,
					  untranslated=untranslated,
                                          percentage=coverage)
                    timestamp.save()
        transaction.commit()

    def status(self, current, total):
        percent = current * 100.0 / total
        sys.stdout.write("%3d%%\r" % percent)
        sys.stdout.flush()

    def add_translations(self):

        TRANSLATION_URL = \
            'http://people.canonical.com/~dpm/data/ubuntu-l10n/' + \
            '{0}-package-stats.json.gz'

        cursor = connection.cursor()

        if settings.DJANGO_1_1:
            psycopg2_cursor = connection.connection.cursor()
        elif settings.DJANGO_1_3:
            psycopg2_cursor = connection.cursor().cursor.cursor

        releases = UbuntuRelease.objects.all()

        #start_time = time.time()
        #sys.stderr.write('Deleting translations table....\n')
        #cursor.execute('TRUNCATE stats_translation;')
        #transaction.commit_unless_managed()
        #print time.time() - start_time, "seconds"
        #sys.stderr.write('Done\n')

        cursor.execute("""
            SET temp_buffers = '128 MB'
        """)

        cursor.execute('''
            CREATE TEMP TABLE stats_temptranslation
                (
                  id serial PRIMARY KEY NOT NULL,
                  translation_domain text NOT NULL,
                  sourcepackage text NOT NULL,
                  code character varying(20) NOT NULL,
                  template_name text NOT NULL,
                  changed integer NOT NULL,
                  template_last_updated timestamp with time zone NOT NULL,
                  unreviewed integer NOT NULL,
                  translated integer NOT NULL,
                  total integer NOT NULL,
                  translation_last_updated timestamp with time zone NOT NULL,
                  release_id integer NOT NULL
                );
        ''')

        for release in releases:
            sys.stderr.write('Saving translation for {0}...\n'.format(
                release))

            # Fetch the translation data for the given release
            translation_url = TRANSLATION_URL.format(release.name)

            data_path = self.download_data(translation_url)
            #data_path = os.path.join(
            #    os.path.dirname(os.path.abspath(__file__)),
            #    '{0}-package-stats.json.gz'.format(release.name))

            fp = gzip.open(data_path, 'rb')
            translation_data = json.load(fp)
            fp.close()

            # Convert the JSON data to CSV in preparation for the Postgresql
            # COPY command
            import StringIO
            csv_file = StringIO.StringIO()

            for translation in translation_data:
                csv_file.write(','.join(
                    [str(item) for item in translation.values()]) + ',' + \
                    str(release.id) + '\n')

            csv_file.seek(0)

            start_time = time.time()

            # Bulk copy the raw data from Launchpad into a temporary table
            # with the Postgresql COPY command. This is extremely faster than
            # doing INSERTs
            psycopg2_cursor.copy_from(
                csv_file,
                'stats_temptranslation',
                sep=',',
                columns=('translation_domain', 'sourcepackage', 'code',
                         'template_name', 'changed',
                         'template_last_updated', 'unreviewed',
                         'translated', 'total', 'translation_last_updated',
                         'release_id'
                         ),
                )

            transaction.commit_unless_managed()
            print time.time() - start_time, "seconds"

            sys.stderr.write('Done CSV import\n')

            add_translations = '''
                INSERT INTO stats_translation
                    (release_id, template_id, language_id, translated)
                SELECT tmp.release_id, t.id, l.id, tmp.translated
                FROM stats_temptranslation tmp
                JOIN stats_language l USING (code)
                JOIN stats_template t ON tmp.template_name = t.template_name
                AND
                tmp.sourcepackage = t.sourcepackage AND
                tmp.translation_domain = t.translation_domain AND
                tmp.release_id = t.release_id AND
                tmp.release_id = (%s)
                ORDER  BY tmp.id;
            '''

            start_time = time.time()

            cursor.execute(add_translations, [release.id])
            transaction.commit_unless_managed()

            print time.time() - start_time, "seconds"
            sys.stderr.write('Done translations import\n')

            ## Add missing templates
            #languages = Language.objects.filter(visible=True)
            #languages_total = len(languages)
            #languages_imported = 0

            #start_time = time.time()
            #for language in languages:
            #    templates = Template.objects.filter(release=release)

            #    for template in templates:
            #        try:
            #            translation = Translation.objects.get(
            #                            template=template,
            #                            language=language,
            #                            release=release)
            #        except Translation.DoesNotExist:
            #            translation = Translation(template=template,
            #                                      language=language,
            #                                      release=release,
            #                                      translated=0)
            #            translation.save()
            #    languages_imported += 1
            #    self.status(languages_imported, languages_total)

            start_time = time.time()
            print 'Starting analyze'
            cursor.execute('''ANALYZE''')
            print time.time() - start_time, "seconds"
            print "Done"

            cursor.execute('''SELECT id FROM stats_translation ORDER BY id DESC LIMIT 1;''')
            nextval = cursor.fetchone()[0] + 1
            print nextval
            cursor.execute('''ALTER SEQUENCE stats_translation_id_seq RESTART %s''', [nextval])

            start_time = time.time()
            languages = Language.objects.filter(visible=True)
            for language in languages:
                add_missing_translations = '''
                    INSERT INTO stats_translation
                        (release_id, template_id, language_id, translated)
                    SELECT
                      %(release_id)s, stats_template.id, %(language_id)s, 0
                    FROM
                      stats_template
                    LEFT JOIN stats_translation ON
                      stats_template.id = stats_translation.template_id AND
                      stats_template.release_id = stats_translation.release_id
                      AND stats_translation.language_id = %(language_id)s
                    WHERE
                      stats_template.release_id = %(release_id)s AND
                      stats_translation.template_id IS NULL
                    ;
                    '''
                cursor.execute(add_missing_translations,
                    {'release_id': release.id, 'language_id': language.id})
            transaction.commit_unless_managed()

            print time.time() - start_time, "seconds"
            sys.stderr.write('Done empty templates import\n')

        cursor.execute('DROP TABLE stats_temptranslation;')
        transaction.commit_unless_managed()

    @transaction.commit_manually
    def add_templates(self):
        EXTRA_SRC_PKGS = ['debian-installer', 'ubiquity',
                      'ubiquity-slideshow-ubuntu',
                      'gfxboot-theme-ubuntu']
        TEMPLATE_URL = \
            'http://people.canonical.com/~dpm/data/ubuntu-l10n/' + \
            '{0}-potemplate-stats.json.gz'

        cursor = connection.cursor()

        releases = UbuntuRelease.objects.all()

        # Doing this is much simpler than trying to keep the Launchpad
        # database and this database in sync
        sys.stderr.write('Deleting templates...')
        cursor.execute('TRUNCATE stats_template CASCADE;')
        transaction.commit_unless_managed()
        #Template.objects.all().delete()
        sys.stderr.write('Done\n')

        blacklist = self.create_blacklist()

        for release in releases:
            # Fetch the template data for the given release
            template_url = TEMPLATE_URL.format(release.name)

            data_path = self.download_data(template_url)

            fp = gzip.open(data_path, 'rb')
            template_data = json.load(fp)

            default_packages = self.get_default_packages(release)

            for lp_template in template_data:
                template = Template(
                        translation_domain=lp_template['translation_domain'],
                        sourcepackage=lp_template['sourcepackage'],
                        template_name=lp_template['template_name'],
                        enabled=lp_template['enabled'],
                        languagepack=lp_template['languagepack'],
                        blacklisted=(lp_template['sourcepackage'],
                                     lp_template['template_name']) in \
                                        blacklist,
                        whitelisted=lp_template['sourcepackage'] in \
                                        EXTRA_SRC_PKGS + default_packages,
                        defaultinstall=lp_template['sourcepackage'] in \
                                        default_packages,
                        priority=lp_template['priority'],
                        total=lp_template['total'],
                        release=release,
                        )
                template.save()

            transaction.commit()

            fp.close()

    def create_blacklist(self):

        blacklist_file = os.path.join(self.CONFIG_PATH, 'blacklist.conf')
        blacklist = []

        for line in open(blacklist_file):
            blk_pkg, blk_pot = line.split('|')
            blacklist.append((blk_pkg.strip(), blk_pot.strip()))

        return blacklist

    def get_default_packages(self, release):
        '''Return a list of source packages that are in a default Ubuntu
        desktop installation, from which we'll be able to determine which
        translation templates are part of the desktop installation

        '''
        distro_seeds =\
            distropackages.DistroPackageSet('ubuntu', release.name, True)
        distro_seeds.fetch()
        return list(distro_seeds.sourcepackages)

    @transaction.commit_manually
    def add_languages(self, lp):
        lp_languages = launchpad.get_languages(lp)

        sys.stderr.write('Saving languages to database...')
        for lp_language in lp_languages:
            try:
                language = Language.objects.get(
                                    name=lp_language.english_name,
                                    code=lp_language.code,
                                    visible=lp_language.visible)
            except Language.DoesNotExist:
                try:
                    language = Language.objects.get(code=lp_language.code)
                    language.name = lp_language.english_name
                except Language.DoesNotExist:
                    language = Language(name=lp_language.english_name,
                                        code=lp_language.code,
                                        visible=lp_language.visible,
                                        translators_count=\
                                            lp_language.translators_count,
                                        )

            language.save()

        transaction.commit()
        sys.stderr.write('Done\n')

    @transaction.commit_manually
    def add_releases(self, lp):
        '''Add Ubuntu releases to the database if they are not present, and
        update their 'active' status'''
        releases = UbuntuRelease.objects.all()
        lp_releases = launchpad.get_releases(lp)

        for lp_release in lp_releases:
            if not releases or not releases.get(name=lp_release.name):
                release = UbuntuRelease(name=lp_release.name,
                                        active=True,
                                        version=lp_release.version)
                release.save()

        transaction.commit()

        for release in releases:
            # See http://djangosnippets.org/snippets/479/ to update only
            # selected fields
            if release.name not in [a.name for a in lp_releases]:
                release.active = False
                release.save()

        transaction.commit()

    def add_countries(self):
        '''This function does not do anything currently, but will be used
        to download country information'''
        COUNTRIES_URL = \
            'http://download.geonames.org/export/dump/countryInfo.txt'

        data_path = self.download_data(COUNTRIES_URL)

    def download_data(self, source):
        '''Downloads a data file from the given source and returns the
        destination path'''
        tempdir = tempfile.mkdtemp()
        atexit.register(shutil.rmtree, tempdir)

        destination = os.path.join(tempdir, os.path.basename(source))

        sys.stderr.write('Fetching data from {0}... '.format(source))
        urllib.urlretrieve(source, destination)
        sys.stderr.write('Done\n')

        return destination

    def handle_noargs(self, **options):
        '''Fetch all data from different sources and import it into the
        database'''
        lp = launchpad.get_launchpad()
        self.add_releases(lp)
        self.add_languages(lp)
        self.add_templates()
        self.add_translations()
        self.add_timestamp()
