#!/usr/bin/env python

# DistroSeeds - Provides a list of packages included in the LiveCD of a
# given distro
#
# Author: David Planella <david.planella@ubuntu.com> and
#   Kyle Nitzsche <kyle.nitzsche@canonical.com>
#
# Copyright (c) 2009, 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Seeds are the lists of packages we want to include in the distribution. We
# have seven primary seeds, minimal, boot, standard, desktop, ship, live, and
# supported, that define what goes into the archive's main component. The
# minimal, boot, standard, desktop, and either ship or live seeds go onto our
# CDs and the supported packages are available from the FTP site.

import sys
import urllib2
import csv
import json


class DistroPackageSet(object):
    """A class that can be used to obtain all source packages resulting from
    expanding the seeds for a given distro.

    These will represent all source packages for all binaries on a default
    installation of the distro.
    """
    MINIMAL_SEEDS = ('boot', 'required', 'minimal', 'standard')
    UBUNTU_DESKTOP_SEEDS = ('desktop-common', 'desktop')
    # TODO: implement Kubuntu
    KUBUNTU_DESKTOP_SEEDS = ('desktop-common', 'desktop')

    GERMINATE_URL = \
        'http://people.canonical.com/~ubuntu-archive/germinate-output'

    def __init__(self, distro_id, distro_codename, json_output = False):
        """Initialize the distro attributes and compile the URL for the
        seeds.
        """
        self.distro_id = distro_id
        self.distro_codename = distro_codename
        self.seeds_distro_url = self._build_distro_url()
        self.seeds = self._build_distro_seeds()
        self.sourcepackages = set()
        self.json_output = json_output

    def fetch(self):
        """Fetch all seeds, combine them, extract the source packages and
        store them as a list in an internal attribute
        """
        for seed in self.seeds:
            seed_url = '{0}/{1}'.format(self.seeds_distro_url, seed)
            seed_opener = urllib2.build_opener()
            try:
                seed_file = seed_opener.open(seed_url)
                seed = seed_file.read()
            except urllib2.URLError, exc:
                logger.critical(
                    "Error opening {0}: {1}\n\
                    Cannot continue.".format(seed_url, exc))
                sys.exit(1)
            else:
                seed_reader = csv.reader(seed.splitlines(), delimiter='|')
                for row in seed_reader:
                    first_row = row[0].strip()
                    if (first_row != 'Package') and not (
                            first_row.startswith('-') or first_row == ''):
                        self.sourcepackages.add(row[1].strip())

    def write(self, path):
        """Writes the current list of source packages from the seeds to
        the given file"""
        with open(path, 'w') as f:
            if not self.json_output:
                f.write(str(self))
            else:
                json.dump((self.distro_codename,
                          list(self.sourcepackages)), f)

    def _build_distro_seeds(self):
        """Returns a list of the required seeds to compute the source
        packages for the given distro_id"""
        if self.distro_id == 'ubuntu':
            distro_seeds = list(self.UBUNTU_DESKTOP_SEEDS)
        elif self.distro_id == 'kubuntu':
            distro_seeds = list(self.KUBUNTU_DESKTOP_SEEDS)

        return list(self.MINIMAL_SEEDS) + distro_seeds

    def _build_distro_url(self):
        """Returns the computed URL where the list of source
        packages for the given distro_id is made available"""
        if self.distro_id == 'ubuntu':
            distro_url = '{0}/{1}.{2}'.format(self.GERMINATE_URL,
                                              self.distro_id,
                                              self.distro_codename)
        elif self.distro_id == 'kubuntu':
            pass

        return distro_url

    def __str__(self):
        """Returns a list of source packages from the seeds when invoking
        the string representation of the class"""
        src_pkg_list = []
        for sourcepackage in self.sourcepackages:
            src_pkg_list.append(sourcepackage)
        src_pkg_list.sort()

        return '\n'.join(src_pkg_list)
