from django.db import models
from django.utils.translation import ugettext as _


class UbuntuRelease(models.Model):
    '''Model for Ubuntu releases
    
    Fields:
    
    name -- the release's codename (e.g. 'precise')
    version -- the release's version (e.g. '12.04')
    active -- whether the release has reached its EOL
    
    '''
    name = models.SlugField(_("Name"), max_length=15)
    version = models.TextField(_("Version"), max_length=10)
    active = models.BooleanField(_("Active"))

    def __unicode__(self):
        return self.name

    # https://docs.djangoproject.com/en/
    # dev/ref/models/instances/#the-permalink-decorator
    @models.permalink
    def get_absolute_url(self):
        return ('name', [self.name])


class Language(models.Model):
    '''Model for natural languages, as stored in Launchpad
    
    Fields:

    name -- the language name in English (e.g. Chinese (Simplified))
    visible -- whether the language is visible in Launchpad
    code -- ISO 639-2 2 or 3-letter code, followed by an optional ISO 3166-1
        country code (e.g. zh_CN)
    team_id -- Ubuntu translations team assigned to this language in Launchpad
    translators_count -- number of contributors to this language, as recorded
        by Launchpad
    
    '''
    name = models.TextField(_("Name"), max_length=70)
    visible = models.BooleanField(_("Visible"))
    code = models.SlugField(_("Code"), max_length=20)
    team_id = models.TextField(_("Team Name"), max_length=70)
    translators_count = models.IntegerField(_("Translators count"))

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('code', [self.code])


class Timestamp(models.Model):
    '''Model to keep track of translation progress over time
    
    Fields:
    
    timestamp -- snapshot date to capture the translation status
    percentage -- indicates how well translated the release is in the given
        language 
    release -- an UbuntuRelease to which the percentage applies
    language -- a Language to which the percentage applies

    '''
    timestamp = models.DateTimeField(_("Timestamp"))
    untranslated = models.IntegerField(_("Untranslated"))
    percentage = models.FloatField(_("Percentage"))
    release = models.ForeignKey(UbuntuRelease)
    language = models.ForeignKey(Language)


class Template(models.Model):
    '''
    
    Fields:
    
    template_name -- the name of the translation template
    sourcepackage -- the source package name the template belongs to
    translation_domain -- the translation domain the template uses at runtime
    total -- translatable messages count
    enabled -- whether the template is enabled (read translatable) in Launchpad
    languagepack -- whether translations for this template will be shipped in
        a language pack
    blacklisted -- whether the template is blacklisted and should not be used
        when calculating the stats
    whitelisted -- whether the template is whitelisted and should
    defaultinstall = models.BooleanField(_("In default installation"))
    priority -- the priority of the package. Higher priority indicates higher
        visibility of its messages in a default desktop installation
    release -- an UbuntuRelease to which the template belongs to
    
    '''
    template_name = models.TextField(_("Name"), max_length=70)
    sourcepackage = models.TextField(_("Source package"), max_length=70)
    translation_domain = models.TextField(_("Domain"), max_length=70)
    total = models.IntegerField(_("Total"))
    enabled = models.BooleanField(_("Enabled"))
    languagepack = models.BooleanField(_("In language pack"))
    blacklisted = models.BooleanField(_("Blacklisted"))
    whitelisted = models.BooleanField(_("Whitelisted"))
    defaultinstall = models.BooleanField(_("In default installation"))
    priority = models.IntegerField(_("Priority"))
    release = models.ForeignKey(UbuntuRelease)

    def is_in_desktop(self, priority):
        '''Calculates if we consider the template to be visible in
        the desktop and thus contribute towards translation stats'''
        if self.enabled and self.whitelisted and not self.blacklisted and \
                self.priority > priority:
            return True

    def __unicode__(self):
        template_id = self.template_name
        return template_id


class Translation(models.Model):
    '''

    Fields:
    
    release --
    template --
    language --
    translated --
    untranslated --

    '''
    release = models.ForeignKey(UbuntuRelease)
    template = models.ForeignKey(Template)
    language = models.ForeignKey(Language)
    translated = models.IntegerField(_("Translated"))

    @property
    def percentage_translated(self):  
        return self.translated * 100.0 / self.template.total

    def is_in_desktop(self, priority):
        if self.template.enabled and self.template.whitelisted and \
                not self.template.blacklisted and \
                self.template.priority > priority:
            return True

    def __unicode__(self):
        return self.template.template_name
