from models import Language, Timestamp, UbuntuRelease, Translation, Template
from django.db.models import Sum
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.conf import settings


def language_list(request, release_name):
    '''Renders the overview page with the language list and the
    corresponding translation percentage figures'''

    release_object = get_object_or_404(UbuntuRelease, name=release_name)

    timestamps = Timestamp.objects.filter(
                        release=release_object).order_by('-timestamp')
    languages = set(map(lambda a: a.language, timestamps))

    # Returns a list of tuples (language, percentage)
    language_list = map(lambda a: (
                a, timestamps.filter(language=a)[0].percentage), languages)

    translated_count = 0
    for language in language_list:
        if language[1] > 80:
            translated_count += 1

    context = {
        'language_list': language_list,
        'release': release_object,
        'translated_count': translated_count,
    }
    return render_to_response('stats/language_list.html', context,
                              RequestContext(request))


def release_list(request):
    '''Renders the list of Ubuntu releases that are active'''

    release_list = UbuntuRelease.objects.filter(active=True)

    context = {
        'release_list': release_list,
    }
    return render_to_response('stats/release_list.html', context,
                              RequestContext(request))


def language_details(request, release_name, language_code):
    '''Renders the language details page, with a diagram showing the
    progress of translations over time, and a list of templates that need
    to be translated'''

    language = get_object_or_404(Language, code=language_code)
    release = get_object_or_404(UbuntuRelease, name=release_name)

    templates = Translation.objects.filter(language=language,
                                           template__release=release,
                                           template__enabled=True,
                                           template__whitelisted=True,
                                           template__blacklisted=False,
                                           template__priority__gt=\
                                            settings.MIN_TEMPLATE_PRIORITY,
                                           )
    total = Template.objects.filter(release=release,
                                    enabled=True,
                                    whitelisted=True,
                                    blacklisted=False,
                                    priority__gt=\
                                        settings.MIN_TEMPLATE_PRIORITY,
                                    ).aggregate(Sum('total'))
    total_translated = templates.aggregate(Sum('translated'))
    total_untranslated = total['total__sum'] - \
                            total_translated['translated__sum']

    language_details = Timestamp.objects.filter(
            language=language, release=release).order_by('timestamp')

    context = {
        'language_details': language_details,
        'language': language,
        'release': release,
        'templates': templates,
        'total': total['total__sum'],
        'translated': total_translated['translated__sum'],
        'untranslated': total_untranslated
    }
    return render_to_response('stats/language_details.html', context,
                              RequestContext(request))
